<?php
/**
 * Class SB_Instagram_API_Connect_Pro
 *
 * Adds support for additional endpoints:
 *
 * - Personal account comments
 * - Business account top and recent hashtags
 * - Business account stories
 * - Business account comments
 * - Business account hashtag IDs
 *
 * @since 5.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

class SB_Instagram_API_Connect_Pro extends SB_Instagram_API_Connect
{
	/**
	 * Sets the url for the API request based on the account information,
	 * type of data needed, and additional parameters
	 *
	 * @param $connected_account
	 * @param $endpoint_slug header or user
	 * @param $params
	 *
	 * @since 5.0
	 */
	protected function set_url( $connected_account, $endpoint_slug, $params ) {
		$account_type = isset( $connected_account['type'] ) ? $connected_account['type'] : 'personal';
		$num = ! empty( $params['num'] ) ? (int)$params['num'] : 33;

		if ( $account_type === 'personal' ) {
			if ( $endpoint_slug === 'header' ) {
				$url = 'https://api.instagram.com/v1/users/' . $connected_account['user_id'] . '?access_token=' . sbi_maybe_clean( $connected_account['access_token'] );
			} elseif ( $endpoint_slug === 'comments' ) {
				$url = 'https://api.instagram.com/v1/media/' . $params['post_id'] . '/comments?access_token=' . sbi_maybe_clean( $connected_account['access_token'] );
			} else {
				$url = 'https://api.instagram.com/v1/users/' . $connected_account['user_id'] . '/media/recent?access_token=' . sbi_maybe_clean( $connected_account['access_token'] );
			}
		} else {
			if ( $endpoint_slug === 'header' ) {
				$url = 'https://graph.facebook.com/' . $connected_account['user_id'] . '?fields=biography,id,username,website,followers_count,media_count,profile_picture_url,name&access_token=' . sbi_maybe_clean( $connected_account['access_token'] );
			} elseif ( $endpoint_slug === 'stories' ) {
				$url = 'https://graph.facebook.com/'.$connected_account['user_id'].'/stories?fields=media_url,caption,id,media_type,permalink,children{media_url,id,media_type,permalink}&limit=100&access_token='.sbi_maybe_clean( $connected_account['access_token'] );
			} elseif ( $endpoint_slug === 'hashtags_top' ) {
				$url = 'https://graph.facebook.com/'.$params['hashtag_id'].'/top_media?user_id='.$connected_account['user_id'].'&fields=media_url,caption,id,media_type,comments_count,like_count,permalink,children{media_url,id,media_type,permalink}&limit='.$num.'&access_token='.sbi_maybe_clean( $connected_account['access_token'] );
			} elseif ( $endpoint_slug === 'hashtags_recent' ) {
				$url = 'https://graph.facebook.com/'.$params['hashtag_id'].'/recent_media?user_id='.$connected_account['user_id'].'&fields=media_url,caption,id,media_type,comments_count,like_count,permalink,children{media_url,id,media_type,permalink}&limit='.$num.'&access_token='.sbi_maybe_clean( $connected_account['access_token'] );
			} elseif ( $endpoint_slug === 'ig_hashtag_search' ) {
				$url = 'https://graph.facebook.com/ig_hashtag_search?user_id='.$connected_account['user_id'].'&q='.urlencode( $params['hashtag'] ).'&access_token='.sbi_maybe_clean( $connected_account['access_token'] );
			} elseif ( $endpoint_slug === 'comments' ) {
				$url = 'https://graph.facebook.com/'.$params['post_id'].'/comments?fields=text,username&access_token='.sbi_maybe_clean( $connected_account['access_token'] );
			} else {
				$num = min( $num, 200 );
				$url = 'https://graph.facebook.com/' . $connected_account['user_id'] . '/media?fields=media_url,thumbnail_url,caption,id,media_type,timestamp,username,comments_count,like_count,permalink,children{media_url,id,media_type,timestamp,permalink,thumbnail_url}&limit='.$num.'&access_token=' . sbi_maybe_clean( $connected_account['access_token'] );
			}
		}

		$this->set_url_from_args( $url );
	}
}