<?php
/**
 * Class SB_Instagram_Parse_Pro
 *
 * @since 5.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

class SB_Instagram_Parse_Pro extends SB_Instagram_Parse
{
	/**
	 * @param $post array
	 *
	 * @return string
	 *
	 * @since 5.0
	 */
	public static function get_username( $post ) {
		if ( ! empty( $post['username'] ) ) {
			return $post['username'];
		} elseif ( ! empty( $post['user']['username'] ) ) {
			return $post['user']['username'];
		} elseif ( isset( $post['data']['username'] ) ) {
			return $post['data']['username'];
		}

		return '';
	}

	public static function get_generic_term( $header_data ) {
		if ( isset( $header_data['term'] ) ) {
			return $header_data['term'];
		} else {
			return '';
		}
	}


	/**
	 * @param $post array
	 *
	 * @return int
	 *
	 * @since 5.0
	 */
	public static function get_likes_count( $post ) {
		if ( ! empty( $post['likes'] ) ) {
			return $post['likes']['count'];
		} elseif ( ! empty( $post['like_count'] ) ) {
			return $post['like_count'];
		}
		return 0;
    }

	/**
	 * @param $post array
	 *
	 * @return int
	 *
	 * @since 5.0
	 */
	public static function get_comments_count( $post ) {
		if ( ! empty( $post['comments']['count'] ) ) {
			return $post['comments']['count'];
		} elseif ( ! empty( $post['comments_count'] ) ) {
			return $post['comments_count'];
		}
		return 0;
	}

	public static function get_item_avatar( $post, $avatars ) {
		if ( empty ( $avatars ) ) {
			return '';
		} else {
			$username = SB_Instagram_Parse_Pro::get_username( $post );
			if ( isset( $avatars[ $username ] ) ) {
				return $avatars[ $username ];
			}
		}
		return '';
	}

	public static function get_lightbox_media_atts( $post ) {
		$return = array(
			'video' => '',
			'carousel' => ''
		);
		if ( isset( $post['videos'] ) ) {
			$return['video'] = $post['videos']['standard_resolution']['url'];
		} elseif ( isset( $post['media_type'] ) && $post['media_type'] === 'VIDEO' && isset( $post['media_url'] ) ) {
			$return['video'] = $post['media_url'];
		} elseif ( isset( $post['media_type'] ) && $post['media_type'] === 'VIDEO' ) {
			$return['video'] = 'missing';
		} else {
			$return['video'] = '';
		}

		if ( SB_Instagram_Parse_Pro::get_media_type( $post ) === 'carousel' ) {
			$carousel_object = SB_Instagram_Parse_Pro::get_carousel_object( $post );
			$return['carousel'] = json_encode( $carousel_object );

			if ( $carousel_object['vid_first'] ) {
				$return['video'] = $carousel_object['data'][0]['media'];
			}
		}

		return $return;
	}

	public static function get_carousel_object( $post ) {
		$car_obj = array(
			'data' => array(),
			'vid_first' => false
		);

		if ( isset( $post['carousel_media'] ) ) {
			$i = 0;
			foreach ( $post['carousel_media'] as $carousel_item ) {
				if ( isset( $carousel_item['images'] ) ) {
					$car_obj['data'][ $i ] = array(
						'type' => 'image',
						'media' => $carousel_item['images']['standard_resolution']['url']
					);
				} elseif ( isset( $carousel_item['videos'] ) ) {
					$car_obj['data'][ $i ] = array(
						'type' => 'video',
						'media' => $carousel_item['videos']['standard_resolution']['url']
					);

					if ( $i === 0 ) {
						$car_obj['vid_first'] = true;
					}
				}

				$i++;
			}
		} elseif ( isset( $post['children'] ) ) {
			$i = 0;
			foreach ( $post['children']['data'] as $carousel_item ) {
				if ( $carousel_item['media_type'] === 'IMAGE' ) {
					$car_obj['data'][ $i ] = array(
						'type' => 'image',
						'media' => $carousel_item['media_url']
					);
				} elseif ( $carousel_item['media_type'] === 'VIDEO' ) {
					if ( isset( $carousel_item['media_url'] ) ) {
						$car_obj['data'][ $i ] = array(
							'type' => 'video',
							'media' => $carousel_item['media_url']
						);

						if ( $i === 0 ) {
							$car_obj['vid_first'] = true;
						}
					} else {
						$permalink = SB_Instagram_Parse::fix_permalink( SB_Instagram_Parse::get_permalink( $post ) );
						$car_obj['data'][ $i ] = array(
							'type' => 'image',
							'media' => $permalink . 'media?size=l'
						);
					}
				}

				$i++;
			}
		}

		return $car_obj;
	}

	public static function get_location_info( $post ) {
		$return = array();
		if ( isset( $post['location'] ) ) {
			$name = ! empty( $post['location'] ) ? $post['location']['name'] : '';
			$return = array(
				'name' => $name,
				'id' => '',
				'longitude' => '',
				'lattitude' => ''
			);
			if ( isset( $post['location']['id'] ) ) {
				$return['id'] = $post['location']['id'];
			}
			if ( isset( $post['location']['longitude'] ) ) {
				$return['longitude'] = $post['location']['longitude'];
			}
			if ( isset( $post['location']['lattitude'] ) ) {
				$return['lattitude'] = $post['location']['lattitude'];
			}
		}

        return $return;
	}

	public static function get_story_data( $header_data ) {
		if ( isset( $header_data['stories'] ) && isset( $header_data['stories'][0] ) ) {
			return $header_data['stories'];
		}
		return '';
	}

	public static function get_post_count( $header_data ) {
		if ( isset( $header_data['data']['counts'] ) ) {
			return $header_data['data']['counts']['media'];
		} elseif ( isset( $header_data['counts'] ) ) {
			return $header_data['counts']['media'];
		} elseif ( isset( $header_data['media_count'] ) ) {
			return $header_data['media_count'];
		}
		return 0;
	}

	public static function get_follower_count( $header_data ) {
		if ( isset( $header_data['data']['counts'] ) ) {
			return $header_data['data']['counts']['followed_by'];
		} elseif ( isset( $header_data['counts'] ) ) {
			return $header_data['counts']['followed_by'];
		} elseif ( isset( $header_data['followers_count'] ) ) {
			return $header_data['followers_count'];
		}
		return 0;
	}

	public static function get_bio( $header_data ) {
		if ( isset( $header_data['data']['bio'] ) ) {
			return $header_data['data']['bio'];
		} elseif ( isset( $header_data['bio'] ) ){
			return $header_data['bio'];
		} elseif ( isset( $header_data['biography'] ) ){
			return $header_data['biography'];
		}
		return '';
	}
}