<?php
/**
 * Custom Feeds for Instagram Header Boxed Template
 * Adds account information and an avatar to the top of the feed
 *
 * @version 5.0 Custom Feeds for Instagram Pro by Smash Balloon
 *
 */

// Don't load directly
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}
$username                = SB_Instagram_Parse_Pro::get_username( $header_data );
$avatar                  = SB_Instagram_Parse_Pro::get_avatar( $header_data, $settings );
$name                    = SB_Instagram_Parse_Pro::get_name( $header_data );
$header_text_color_style = SB_Instagram_Display_Elements_Pro::get_header_text_color_styles( $settings ); // style="color: #517fa4;"

// Pro Elements
$type_class        = SB_Instagram_Display_Elements_Pro::get_feed_type_class( $settings );
$size_class        = SB_Instagram_Display_Elements_Pro::get_header_size_class( $settings );
$stories_delay     = SB_Instagram_Display_Elements_Pro::get_stories_delay( $settings );
$story_data        = SB_Instagram_Parse_Pro::get_story_data( $header_data );
$should_show_story = $story_data !== '' ? SB_Instagram_Display_Elements_Pro::should_show_element( 'headerstory', $settings ) : false;
$story_data_att    = $should_show_story ? ' data-story-wait="'. (int)$stories_delay . '" data-story-data="' . esc_attr( json_encode( $story_data ) ) . '" data-story-avatar="'.$avatar.'"' : '';
$post_count        = SB_Instagram_Parse_Pro::get_post_count( $header_data );
$follower_count    = SB_Instagram_Parse_Pro::get_follower_count( $header_data );
$bio               = SB_Instagram_Parse_Pro::get_bio( $header_data );

// Pro Styles
$should_show_bio = $bio !== '' ? SB_Instagram_Display_Elements_Pro::should_show_element( 'headerbio', $settings ) : false;
$bio_class       = ! $should_show_bio ? ' sbi_no_bio' : '';

// Boxed Header Specific
$follow_button_text = __( $settings['followtext'], 'instagram-feed' );
$header_style       = SB_Instagram_Display_Elements_Pro::get_boxed_header_styles( $settings ); // style="background: #517fa4;"
$header_bar_style   = SB_Instagram_Display_Elements_Pro::get_header_bar_styles( $settings ); // style="background: #eeeeee;"
$header_info_style  = SB_Instagram_Display_Elements_Pro::get_header_info_styles( $settings ); // style="color: #517fa4;"
$follow_btn_style   = SB_Instagram_Display_Elements_Pro::get_follow_styles( $settings ); // style="background: rgb();color: rgb();"
$follow_btn_classes = strpos( $follow_btn_style, 'background' ) !== false ? ' sbi_custom' : '';
?>
<div class="sb_instagram_header sbi_header_style_boxed <?php echo $type_class; ?><?php echo $size_class; ?>" <?php echo $header_style; ?>
    <?php echo $story_data_att; ?>>
<a href="https://www.instagram.com/<?php echo urlencode( $username ); ?>" target="_blank" rel="noopener"
   title="@<?php echo esc_attr( $username ); ?>"
   class="sbi_header_link">
    <div class="sbi_header_text<?php echo $bio_class; ?>">
        <h3 <?php echo $header_text_color_style; ?>><?php echo esc_html( $username ); ?></h3>
		<?php if ( $should_show_bio ) : ?>
            <p class="sbi_bio" <?php echo $header_text_color_style; ?>><?php echo esc_html( $bio ); ?></p>
		<?php endif; ?>
    </div>
    <div class="sbi_header_img" data-avatar-url="<?php echo esc_attr( SB_Instagram_Parse::get_avatar( $header_data ) ); ?>">
        <div class="sbi_header_img_hover"><?php echo SB_Instagram_Display_Elements_Pro::get_icon( 'newlogo', $icon_type ); ?></div>
        <img src="<?php echo esc_url( $avatar ); ?>"
             alt="<?php echo esc_attr( $name ); ?>" width="50" height="50">
    </div>
    <div class="sbi_header_bar" <?php echo $header_bar_style; ?>>
        <p class="sbi_bio_info" <?php echo $header_info_style; ?>>
            <span class="sbi_posts_count"><?php echo SB_Instagram_Display_Elements_Pro::get_icon( 'photo', $icon_type ) . number_format_i18n( $post_count, 0 ); ?></span>
            <span class="sbi_followers"><?php echo SB_Instagram_Display_Elements_Pro::get_icon( 'user', $icon_type ) . number_format_i18n( $follower_count, 0 ); ?></span>
        </p>
        <a class="sbi_header_follow_btn<?php echo $follow_btn_classes; ?>"
           href="https://instagram.com/<?php echo urlencode( $username ); ?>" target="_blank" rel="noopener"
			<?php echo $follow_btn_style; ?>>
			<?php echo SB_Instagram_Display_Elements_Pro::get_icon( 'instagram', $icon_type ); ?>
            <span><?php echo esc_html( $follow_button_text ); ?></span>
        </a>
    </div>
</a>
</div>